@@ -26,6 +26,7 @@
 #include "seahorse-context.h"
 #include "seahorse-key.h"
 #include "seahorse-util.h"
+#include "seahorse-key-dialogs.h"
 
 #define PROMPT "keyedit.prompt"
 #define QUIT "quit"
@@ -76,7 +77,7 @@ seahorse_ops_key_delete (SeahorseContext *sctx, SeahorseKey *skey)
 gboolean
 seahorse_ops_key_generate (SeahorseContext *sctx, const gchar *name,
 			   const gchar *email, const gchar *comment,
-			   const gchar *passphrase, const SeahorseKeyType type,
+			   const gchar *pass, const SeahorseKeyType type,
 			   const guint length, const time_t expires)
 {
         gchar *common, *key_type, *start, *parms;
@@ -84,10 +85,8 @@ seahorse_ops_key_generate (SeahorseContext *sctx, const gchar *name,
 	const gchar *expires_date;
 	
 	g_return_val_if_fail (sctx != NULL && SEAHORSE_IS_CONTEXT (sctx), FALSE);
-	g_return_val_if_fail (name != NULL && !g_str_equal (name, ""), FALSE);
-	g_return_val_if_fail (email != NULL && seahorse_ops_key_check_email (email), FALSE);
-	g_return_val_if_fail (comment != NULL && !g_str_equal (comment, ""), FALSE);
-	g_return_val_if_fail (passphrase != NULL && !g_str_equal (passphrase, ""), FALSE);
+	g_return_val_if_fail (strlen (name) >= 5, FALSE);
+	g_return_val_if_fail (strlen (pass) > 0, FALSE);
 	
 	/* Check lengths for each type */
 	switch (type) {
@@ -111,16 +110,23 @@ seahorse_ops_key_generate (SeahorseContext *sctx, const gchar *name,
 		expires_date = "0";
 	
 	/* Common xml */
+	common = g_strdup_printf ("Name-Real: %s\nExpire-Date: %s\nPassphrase: %s\n"
+		"</GnupgKeyParms>", name, expires_date, pass);
+	if (strlen (email) > 0)
+		common = g_strdup_printf ("Name-Email: %s\n%s", email, common);
+	if (strlen (comment) > 0)
+		common = g_strdup_printf ("Name-Comment: %s\n%s", comment, common);
+	
+	/*
 	common = g_strdup_printf ("Name-Real: %s\nName-Comment: %s\nName-Email: %s\n"
 		"Expire-Date: %s\nPassphrase: %s\n</GnupgKeyParms>", name, comment, email,
-		expires_date, passphrase);
+		expires_date, passphrase);*/
 	
 	if (type == RSA_SIGN)
 		key_type = "Key-Type: RSA";
 	else
 		key_type = "Key-Type: DSA";
 	
-	/* More common xml */
 	start = g_strdup_printf ("<GnupgKeyParms format=\"internal\">\n%s\nKey-Length: ", key_type);
 	
 	/* Subkey xml */
@@ -714,6 +720,23 @@ edit_pass_transit (guint current_state, GpgmeStatusCode status,
 	return next_state;
 }
 
+static const gchar*
+edit_pass_get (SeahorseContext *sctx, const gchar *desc, gpointer *data)
+{
+	static gint count = 0;
+	
+	if (!desc)
+		return NULL;
+	if (desc[0] == 'E')
+		count++;
+	if (count == 1)
+		return seahorse_passphrase_get (sctx, desc, data);
+	else {
+		count = 0;
+		return seahorse_change_passphrase_get (sctx, desc, data);
+	}
+}
+
 /**
  * seahorse_ops_key_change_pass:
  * @sctx: Current context
@@ -728,18 +751,16 @@ gboolean
 seahorse_ops_key_change_pass (SeahorseContext *sctx, SeahorseKey *skey)
 {
 	SeahorseEditParm *parms;
-	GpgmePassphraseCb callback = NULL;
+	gboolean success;
 	
 	g_return_val_if_fail (sctx != NULL && SEAHORSE_IS_CONTEXT (sctx), FALSE);
 	g_return_val_if_fail (skey != NULL && SEAHORSE_IS_KEY (skey), FALSE);
 	
-	/* Check if have passphrase callback */
-	gpgme_get_passphrase_cb (sctx->ctx, &callback, NULL);
-	g_return_val_if_fail (callback != NULL, FALSE);
-	
 	parms = seahorse_edit_parm_new (PASS_START, edit_pass_action, edit_pass_transit, NULL);
 	
-	return edit_key (sctx, skey, parms, _("Passphrase Change"), SKEY_CHANGE_PASS);
+	gpgme_set_passphrase_cb (sctx->ctx, (GpgmePassphraseCb)edit_pass_get, sctx);
+	success = edit_key (sctx, skey, parms, _("Passphrase Change"), SKEY_CHANGE_PASS);
+	gpgme_set_passphrase_cb (sctx->ctx, (GpgmePassphraseCb)seahorse_passphrase_get, sctx);
 }
 
 /**
@@ -897,7 +918,6 @@ seahorse_ops_key_add_uid (SeahorseContext *sctx, SeahorseKey *skey,
 	g_return_val_if_fail (sctx != NULL && SEAHORSE_IS_CONTEXT (sctx), FALSE);
 	g_return_val_if_fail (skey != NULL && SEAHORSE_IS_KEY (skey), FALSE);
 	g_return_val_if_fail (name != NULL && strlen (name) >= 5, FALSE);
-	g_return_val_if_fail (email != NULL && seahorse_ops_key_check_email (email), FALSE);
 	
 	uid_parm = g_new (UidParm, 1);
 	uid_parm->name = name;
@@ -1643,16 +1663,17 @@ gboolean
 seahorse_ops_key_add_revoker (SeahorseContext *sctx, SeahorseKey *skey)
 {
 	SeahorseEditParm *parms;
-	SeahorseKey *signer;
+	SeahorseKeyPair *skpair;
 	
 	g_return_val_if_fail (sctx != NULL && SEAHORSE_IS_CONTEXT (sctx), FALSE);
 	g_return_val_if_fail (skey != NULL && SEAHORSE_IS_KEY (skey), FALSE);
 	
-	signer = seahorse_context_get_last_signer (sctx);
-	g_return_val_if_fail (signer != NULL, FALSE);
+	skpair = seahorse_context_get_default_key (sctx);
+	g_return_val_if_fail (skpair != NULL, FALSE);
 	
 	parms = seahorse_edit_parm_new (ADD_REVOKER_START, add_revoker_action,
-		add_revoker_transit, (gpointer)seahorse_key_get_keyid (signer, 0));
+		add_revoker_transit, (gpointer)gpgme_key_get_string_attr (
+		SEAHORSE_KEY (skpair)->key, GPGME_ATTR_FPR, NULL, 0));
 	
 	return edit_key (sctx, skey, parms, _("Add Revoker"), SKEY_CHANGE_REVOKER);
 }
